/* Copyright (C) 2002-2003 RealVNC Ltd.  All Rights Reserved.
 *    
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */

// -=- HTTPServer.h

// Single-threaded HTTP server implementation.
// All I/O is handled by the processSocketEvent routine,
// which is called by the main-loop of the VNC server whenever
// there is an event on an HTTP socket.

#ifndef __RFB_HTTP_SERVER_H__
#define __RFB_HTTP_SERVER_H__

#include <list>

#include <rdr/MemInStream.h>
#include <rfb/UpdateTracker.h>
#include <rfb/Configuration.h>
#include <network/Socket.h>

namespace rfb {

  class HTTPServer : public network::SocketServer {
  public:
    // -=- Constructors

    // - HTTPServer(files)
    //   Create an HTTP server which will use the getFile method
    //   to satisfy HTTP GET requests.
    HTTPServer();

    virtual ~HTTPServer();

    // -=- Client management

    // - Run a client connection on the supplied socket
    //   This causes the server to perform HTTP protocol on the
    //   supplied socket.
    //   NB:  The server assumes ownership of the Socket object.
    virtual bool addClient(network::Socket* sock);

    // -=- Event processing methods

    // - Process an input event on a particular Socket
    //   The platform-specific side of the server implementation calls
    //   this method whenever data arrives on one of the active
    //   network sockets.
    //   The method returns true if the Socket is still in use by the
    //   server, or false if it is no longer required and should be
    //   deleted.
    //   NB:  If false is returned then the Socket is deleted and must
    //   not be accessed again!

    virtual bool processSocketEvent(network::Socket* sock);

    // getSockets() gets a list of sockets.  This can be used to generate an
    // fd_set for calling select().

    virtual void getSockets(std::list<network::Socket*>* sockets);

    virtual int checkIdleTimeouts();


    // -=- File interface

    // - getFile is passed the path portion of a URL and returns a
    //   pointer to the data to return, its type and its length.
    //   If the file is not available then false is returned.
    //   Overridden getFile functions should call the default version
    //   if they do not recognise a path name.
    //   NB: The caller assumes ownership of the returned InStream.

    virtual rdr::InStream* getFile(const char* name);

  protected:
    class Session;
    std::list<Session*> sessions;
  };
}

#endif

