/* Copyright (C) 2002-2003 RealVNC Ltd.  All Rights Reserved.
 *    
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */

// -=- LogWriter.cxx - client-side logging interface

#include <string.h>
#ifdef WIN32
#define strcasecmp _stricmp
#endif

#include <rfb/LogWriter.h>

using namespace rfb;

LogWriter::LogWriter(const char* name) : m_name(name), m_level(0), m_log(0), m_next(log_writers) {
  log_writers = this;
}

LogWriter::~LogWriter() {
  // *** Should remove this logger here!
}

void LogWriter::setLog(Logger *logger) {
  m_log = logger;
}

void LogWriter::setLevel(int level) {
  m_level = level;
}

void
LogWriter::listLogWriters(int width) {
  // *** make this respect width...
  LogWriter* current = log_writers;
  printf("  ");
  while (current) {
    printf("%s", current->m_name);
    current = current->m_next;
    if (current) printf(", ");
  }
  printf("\n");
}

LogWriter* LogWriter::log_writers;

LogWriter*
LogWriter::getLogWriter(const char* name) {
  LogWriter* current = log_writers;
  while (current) {
    if (strcasecmp(name, current->m_name) == 0) return current;
      current = current->m_next;
    }
  return 0;
}

bool LogWriter::setLogParams(const char* params) {
  char logwriter_name[256], logger_name[256];
  int level;
  int matched = sscanf(params, "%255[^:]:%255[^:]:%d",
                       logwriter_name, logger_name, &level);
  if (matched < 3) {
    fprintf(stderr,"not all parameters matched: %d\n",matched);
    return false;
  }
  Logger* logger = 0;
  if (strcmp("", logger_name) != 0) {
    logger = Logger::getLogger(logger_name);
    if (!logger) fprintf(stderr,"no logger found! %s\n",logger_name);
  }
  if (strcmp("*", logwriter_name) == 0) {
    LogWriter* current = log_writers;
    while (current) {
      current->setLog(logger);
      current->setLevel(level);
      current = current->m_next;
    }
    return true;
  } else {
    LogWriter* logwriter = getLogWriter(logwriter_name);
    if (!logwriter) {
      fprintf(stderr,"no logwriter found! %s\n",logwriter_name);
    } else {
      logwriter->setLog(logger);
      logwriter->setLevel(level);
      return true;
    }
  }
  return false;
}
