/* Copyright (C) 2002-2003 RealVNC Ltd.  All Rights Reserved.
 *    
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
//
// SSecurity - class on the server side for handling security handshaking.  A
// derived class for a particular security type overrides the processMsg()
// method.  processMsg() is called first when the security type has been
// decided on, and will keep being called whenever there is data to read from
// the client until either it returns false, indicating authentication/security
// failure, or it returns with done set to true, to indicate success.
//
// processMsg() must never block (or at least must never block until the client
// has been authenticated) - this is to prevent denial of service attacks.
// Note that the first time processMsg() is called, there is no guarantee that
// there is any data to read from the SConnection's InStream, but subsequent
// calls guarantee there is at least one byte which can be read without
// blocking.

#ifndef __RFB_SSECURITY_H__
#define __RFB_SSECURITY_H__

namespace rfb {

  class SConnection;

  class SSecurity {
  public:
    virtual ~SSecurity() {}
    virtual bool processMsg(SConnection* sc, bool* done)=0;
    virtual void destroy() { delete this; }
  };

  // SSecurityFactory creates new SSecurity instances for
  // particular security types.
  // The instances must be destroyed by calling destroy()
  // on them when done.
  class SSecurityFactory {
  public:
    virtual ~SSecurityFactory() {}
    virtual SSecurity* getSSecurity(int secType)=0;
  };

}
#endif
