/* Copyright (C) 2002-2003 RealVNC Ltd.  All Rights Reserved.
 *    
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
#include <rdr/OutStream.h>
#include <rfb/ImageGetter.h>
#include <rfb/encodings.h>
#include <rfb/ConnParams.h>
#include <rfb/SMsgWriter.h>
#include <rfb/ZRLEEncoder.h>

using namespace rfb;

#define EXTRA_ARGS ImageGetter* ig
#define GET_IMAGE_INTO_BUF(r,buf) ig->getImage(buf, r);
#define BPP 8
#include <rfb/zrleEncode.h>
#undef BPP
#define BPP 16
#include <rfb/zrleEncode.h>
#undef BPP
#define BPP 32
#include <rfb/zrleEncode.h>
#define CPIXEL 24A
#include <rfb/zrleEncode.h>
#undef CPIXEL
#define CPIXEL 24B
#include <rfb/zrleEncode.h>
#undef CPIXEL
#undef BPP

Encoder* ZRLEEncoder::create(SMsgWriter* writer)
{
  return new ZRLEEncoder(writer);
}

ZRLEEncoder::ZRLEEncoder(SMsgWriter* writer_) : writer(writer_)
{
}

ZRLEEncoder::~ZRLEEncoder()
{
}

void ZRLEEncoder::writeRect(const Rect& r, ImageGetter* ig)
{
  rdr::U8* imageBuf = writer->getImageBuf(64 * 64 * 4 + 4);
  mos.clear();

  switch (writer->bpp()) {
  case 8:  zrleEncode8(r, &mos, &zos, imageBuf, ig);  break;
  case 16: zrleEncode16(r, &mos, &zos, imageBuf, ig); break;
  case 32:
    {
      const PixelFormat& pf = writer->getConnParams()->pf();

      bool fitsInLS3Bytes = ((pf.redMax   << pf.redShift)   < (1<<24) &&
                             (pf.greenMax << pf.greenShift) < (1<<24) &&
                             (pf.blueMax  << pf.blueShift)  < (1<<24));

      bool fitsInMS3Bytes = (pf.redShift   > 7  &&
                             pf.greenShift > 7  &&
                             pf.blueShift  > 7);

      if ((fitsInLS3Bytes && !pf.bigEndian) ||
          (fitsInMS3Bytes && pf.bigEndian))
      {
        zrleEncode24A(r, &mos, &zos, imageBuf, ig);
      }
      else if ((fitsInLS3Bytes && pf.bigEndian) ||
               (fitsInMS3Bytes && !pf.bigEndian))
      {
        zrleEncode24B(r, &mos, &zos, imageBuf, ig);
      }
      else
      {
        zrleEncode32(r, &mos, &zos, imageBuf, ig);
      }
      break;
    }
  }

  writer->startRect(r, encodingZRLE);
  rdr::OutStream* os = writer->getOutStream();
  os->writeU32(mos.length());
  os->writeBytes(mos.data(), mos.length());
  writer->endRect();
}
