/* Copyright (C) 2002-2003 RealVNC Ltd.  All Rights Reserved.
 *    
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
#include <string.h>
#ifdef _WIN32
#define strcasecmp _stricmp
#endif
#include <rfb/secTypes.h>
#include <rfb/util.h>

int rfb::secTypeNum(const char* name)
{
  if (strcasecmp(name, "None") == 0)       return secTypeNone;
  if (strcasecmp(name, "VncAuth") == 0)    return secTypeVncAuth;
  return secTypeInvalid;
}

const char* rfb::secTypeName(int num)
{
  switch (num) {
  case secTypeNone:       return "None";
  case secTypeVncAuth:    return "VncAuth";
  default:                return "[unknown secType]";
  }
}

std::list<int> rfb::parseSecTypes(const char* types)
{
  std::list<int> result;
  int start = 0;
  for (int i=0; i<=(int)strlen(types); i++) {
    if ((types[i] == ',') || (types[i] == 0)) {
      if (i-start > 0) {
        CharArray tmp(i-start+1);
        memcpy(tmp.buf, &types[start], i-start);
        tmp.buf[i-start] = 0;
        int type = secTypeNum(tmp.buf);
        if (type != secTypeInvalid)
          result.push_back(type);
      }
      start = i+1;
    }
  }
  return result;
}
